{******************************************************************************}
{*    "DirectX.    Delphi".                  *}
{*  BHV. www.bhv.ru                                               *}
{*  .. softgl@chat.ru                                                *}
{******************************************************************************}
unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  AppEvnts,
  DirectXGraphics, DXGUtils;

type
  TfrmD3D = class(TForm)
    ApplicationEvents1: TApplicationEvents;
    procedure FormCreate(Sender: TObject);
    procedure ApplicationEvents1Minimize(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure ApplicationEvents1Idle(Sender: TObject; var Done: Boolean);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ApplicationEvents1Restore(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    FD3D            : IDIRECT3D8;
    FD3DDevice      : IDIRECT3DDEVICE8;
    FD3DVB          : IDIRECT3DVERTEXBUFFER8;
    FD3Texture1     : IDIRECT3DTEXTURE8;
    FD3Texture2     : IDIRECT3DTEXTURE8;
    FD3Texture3     : IDIRECT3DTEXTURE8;
    FD3Texture4     : IDIRECT3DTEXTURE8;
    FD3Texture5     : IDIRECT3DTEXTURE8;
    FD3Texture6     : IDIRECT3DTEXTURE8;
    FActive         : BOOL;
    ThisTickCount   : DWORD;
    LastTickCount   : DWORD;
    function Render : HRESULT;
    function InitD3D : HRESULT;
    function InitVB : HRESULT;
    function InitTexture (var FD3Texture : IDIRECT3DTEXTURE8;
                          const FileName : String) : HRESULT;
    procedure ReadData;
    procedure DrawScene;
    procedure Cleanup;
    procedure ErrorOut (const Caption : PChar; const hError : HRESULT);
  public
    { Public declarations }
  end;

type
  TCUSTOMVERTEX = packed record
    X, Y, Z : Single;
    nX, nY, nZ : Single;
    U, V : Single;
  end;

const
  D3DFVF_CUSTOMVERTEX = D3DFVF_XYZ or D3DFVF_NORMAL or D3DFVF_TEX1;

var
  frmD3D: TfrmD3D;
  Frames : Integer = 0;

  AngleZ : Single = 0;
  Angle : Single = 0;

implementation

{$R *.DFM}

procedure TfrmD3D.Cleanup;
begin
  if Assigned (FD3Texture6) then begin
     FD3Texture6._Release;
     FD3Texture6 := nil;
  end;

  if Assigned (FD3Texture5) then begin
     FD3Texture5._Release;
     FD3Texture5 := nil;
  end;

  if Assigned (FD3Texture4) then begin
     FD3Texture4._Release;
     FD3Texture4 := nil;
  end;

  if Assigned (FD3Texture3) then begin
     FD3Texture3._Release;
     FD3Texture3 := nil;
  end;

  if Assigned (FD3Texture2) then begin
     FD3Texture2._Release;
     FD3Texture2 := nil;
  end;

  if Assigned (FD3Texture1) then begin
     FD3Texture1._Release;
     FD3Texture1 := nil;
  end;

  if Assigned (FD3DVB) then begin
     FD3DVB._Release;
     FD3DVB := nil;
  end;

  if Assigned (FD3DDevice) then begin
     FD3DDevice._Release;
     FD3DDevice := nil;
  end;

  if Assigned (FD3D) then begin
     FD3D._Release;
     FD3D := nil;
  end;
end;

procedure TfrmD3D.ErrorOut (const Caption : PChar; const hError : HRESULT);
begin
  FActive := False;
  Cleanup;
  MessageBox (Handle, PChar(DXGErrorString (hError)), Caption, 0);
end;

function TfrmD3D.InitD3D : HRESULT;
var
  d3ddm : TD3DDISPLAYMODE;
  d3dpp : TD3DPRESENT_PARAMETERS;
  hRet : HRESULT;
begin
  FD3Texture6 := nil;
  FD3Texture5 := nil;
  FD3Texture4 := nil;
  FD3Texture3 := nil;
  FD3Texture2 := nil;
  FD3Texture1 := nil;
  FD3DDevice := nil;
  FD3DVB := nil;
  FD3D := nil;

  FD3D := Direct3DCreate8(D3D_SDK_VERSION);
  if FD3D = nil then begin
     Result := E_FAIL;
     Exit;
  end;

  hRet := FD3D.GetAdapterDisplayMode(D3DADAPTER_DEFAULT, d3ddm);
  if FAILED(hRet) then begin
     Result := hRet;
     Exit;
  end;

  ZeroMemory(@d3dpp, SizeOf(d3dpp));
  with d3dpp do begin
       Windowed := True;
       SwapEffect := D3DSWAPEFFECT_DISCARD;
       BackBufferFormat := d3ddm.Format;
       EnableAutoDepthStencil := True;
       AutoDepthStencilFormat := D3DFMT_D16;
  end;

  Result := FD3D.CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, Handle,
                              D3DCREATE_SOFTWARE_VERTEXPROCESSING,
                              d3dpp, FD3DDevice);
end;

function TfrmD3D.Render : HRESULT;
var
  hRet : HRESULT;
begin
  if FD3DDevice = nil then begin
     Result := E_FAIL;
     Exit;
  end;

  hRet := FD3DDevice.Clear(0, nil, D3DCLEAR_TARGET or D3DCLEAR_ZBUFFER,
                           0, 1.0, 0);
  if FAILED(hRet) then begin
     Result := hRet;
     Exit;
  end;

  hRet := FD3DDevice.BeginScene;
  if FAILED(hRet) then begin
     Result := hRet;
     Exit;
  end;

  with FD3DDevice do begin
       SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);
       SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE);
       SetTextureStageState(0, D3DTSS_COLOROP, D3DTA_TEXTURE);
  end;

  DrawScene;

  FD3DDevice.SetTexture(0, FD3Texture1);

  hRet := FD3DDevice.EndScene;
  if FAILED(hRet) then begin
     Result := hRet;
     Exit;
  end;

  Result := FD3DDevice.Present(nil, nil, 0, nil);
end;

procedure TfrmD3D.FormCreate(Sender: TObject);
var
  hRet : HRESULT;
  matView, matProj : TD3DMatrix;
begin
  ShowCursor (False);

  hRet := InitD3D;
  if Failed (hRet) then ErrorOut ('InitD3D', hRet);
  hRet := InitVB;
  if Failed (hRet) then ErrorOut ('InitVertex', hRet);
  hRet := InitTexture (FD3Texture1, 'skylf.bmp');
  if Failed (hRet) then ErrorOut ('InitTexture', hRet);

  hRet := InitTexture (FD3Texture2, 'skyft.bmp');
  if Failed (hRet) then ErrorOut ('InitTexture', hRet);

  hRet := InitTexture (FD3Texture3, 'skyrt.bmp');
  if Failed (hRet) then ErrorOut ('InitTexture', hRet);

  hRet := InitTexture (FD3Texture4, 'skybk.bmp');
  if Failed (hRet) then ErrorOut ('InitTexture', hRet);

  hRet := InitTexture (FD3Texture5, 'skyup.bmp');
  if Failed (hRet) then ErrorOut ('InitTexture', hRet);

  hRet := InitTexture (FD3Texture6, 'skydn.bmp');
  if Failed (hRet) then ErrorOut ('InitTexture', hRet);

  SetViewMatrix(matView, D3DVector(0, 0, 0), D3DVector(0, 1, 0), D3DVector(0, 1, 0));
  FD3DDevice.SetTransform(D3DTS_VIEW, matView);

  SetProjectionMatrix(matProj, 1, 1, 0.1, 1.1);
  FD3DDevice.SetTransform(D3DTS_PROJECTION, matProj);
end;

procedure TfrmD3D.ApplicationEvents1Minimize(Sender: TObject);
begin
  FActive := False;
end;

procedure TfrmD3D.FormActivate(Sender: TObject);
begin
  LastTickCount := GetTickCount;
  FActive := True;
end;

procedure TfrmD3D.ApplicationEvents1Idle(Sender: TObject;
  var Done: Boolean);
var
  hRet : HRESULT;
begin
  if FActive then begin
     Inc (Frames);
     hRet := Render;
     if FAILED(hRet) then begin
        FActive := False;
        ErrorOut ('Render', hRet);
        Exit;
     end;

     ReadData;

     ThisTickCount := GetTickCount;
     if ThisTickCount - LastTickCount > 15 then begin
        Caption := 'FPS = ' + Format('%6.2f',
                                 [frames * 1000 / (ThisTickCount - LastTickCount)]);
        Frames := 0;

{        AngleZ := AngleZ + 0.05;
        if AngleZ > 2 * Pi then AngleZ := AngleZ - 2 * Pi;}

        LastTickCount := GetTickCount;
     end;
  end;

  Done := False;
end;

procedure TfrmD3D.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_ESCAPE then Close;
end;

procedure TfrmD3D.ApplicationEvents1Restore(Sender: TObject);
begin
  FActive := True;
end;

procedure TfrmD3D.FormDestroy(Sender: TObject);
begin
  Cleanup
end;

function TfrmD3D.InitVB : HRESULT;
var
  Vertices : ^TCustomVertex;
  hRet : HRESULT;
begin
  hRet := FD3DDevice.CreateVertexBuffer(36 * SizeOf(TCustomVertex), 0,
                                        D3DFVF_CUSTOMVERTEX,
                                        D3DPOOL_DEFAULT, FD3DVB);
  if Failed(hRet) then begin
     Result := hRet;
     Exit;
  end;

  hRet := FD3DDevice.SetStreamSource(0, FD3DVB, SizeOf(TCustomVertex));
  if Failed(hRet) then begin
     Result := hRet;
     Exit;
  end;

  hRet := FD3DDevice.SetVertexShader(D3DFVF_CUSTOMVERTEX);
  if Failed(hRet) then begin
     Result := hRet;
     Exit;
  end;

  hRet := FD3DVB.Lock(0, 36 * SizeOf(TCustomVertex), PByte(Vertices), 0);
  if Failed(hRet) then begin
     Result := hRet;
     Exit;
  end;

  Vertices.X := -0.5;
  Vertices.Y := -0.5;
  Vertices.Z := -0.5;
  Vertices.nX := -1.0;
  Vertices.nY := 0.0;
  Vertices.nZ := 0.0;
  Vertices.U := 1.0;
  Vertices.V := 0.0;
  Inc(Vertices);

  Vertices.X := -0.5;
  Vertices.Y :=  0.5;
  Vertices.Z := -0.5;
  Vertices.nX := -1.0;
  Vertices.nY := 0.0;
  Vertices.nZ := 0.0;
  Vertices.U := 0.0;
  Vertices.V := 0.0;
  Inc(Vertices);

  Vertices.X := -0.5;
  Vertices.Y := -0.5;
  Vertices.Z :=  0.5;
  Vertices.nX := -1.0;
  Vertices.nY := 0.0;
  Vertices.nZ := 0.0;
  Vertices.U := 1.0;
  Vertices.V := 1.0;
  Inc(Vertices);

  Vertices.X := -0.5;
  Vertices.Y := -0.5;
  Vertices.Z :=  0.5;
  Vertices.nX := -1.0;
  Vertices.nY := 0.0;
  Vertices.nZ := 0.0;
  Vertices.U := 1.0;
  Vertices.V := 1.0;
  Inc(Vertices);

  Vertices.X := -0.5;
  Vertices.Y :=  0.5;
  Vertices.Z := -0.5;
  Vertices.nX := -1.0;
  Vertices.nY := 0.0;
  Vertices.nZ := 0.0;
  Vertices.U := 0.0;
  Vertices.V := 0.0;
  Inc(Vertices);

  Vertices.X := -0.5;
  Vertices.Y :=  0.5;
  Vertices.Z :=  0.5;
  Vertices.nX := -1.0;
  Vertices.nY := 0.0;
  Vertices.nZ := 0.0;
  Vertices.U := 0.0;
  Vertices.V := 1.0;
  Inc(Vertices);

  Vertices.X := -0.5;
  Vertices.Y :=  0.5;
  Vertices.Z := -0.5;
  Vertices.nX := 0.0;
  Vertices.nY := 1.0;
  Vertices.nZ := 0.0;
  Vertices.U := 1.0;
  Vertices.V := 0.0;
  Inc(Vertices);

  Vertices.X :=  0.5;
  Vertices.Y :=  0.5;
  Vertices.Z := -0.5;
  Vertices.nX := 0.0;
  Vertices.nY := 1.0;
  Vertices.nZ := 0.0;
  Vertices.U := 0.0;
  Vertices.V := 0.0;
  Inc(Vertices);

  Vertices.X := -0.5;
  Vertices.Y :=  0.5;
  Vertices.Z :=  0.5;
  Vertices.nX := 0.0;
  Vertices.nY := 1.0;
  Vertices.nZ := 0.0;
  Vertices.U := 1.0;
  Vertices.V := 1.0;
  Inc(Vertices);

  Vertices.X := -0.5;
  Vertices.Y :=  0.5;
  Vertices.Z :=  0.5;
  Vertices.nX := 0.0;
  Vertices.nY := 1.0;
  Vertices.nZ := 0.0;
  Vertices.U := 1.0;
  Vertices.V := 1.0;
  Inc(Vertices);

  Vertices.X :=  0.5;
  Vertices.Y :=  0.5;
  Vertices.Z := -0.5;
  Vertices.nX := 0.0;
  Vertices.nY := 1.0;
  Vertices.nZ := 0.0;
  Vertices.U := 0.0;
  Vertices.V := 0.0;
  Inc(Vertices);

  Vertices.X := 0.5;
  Vertices.Y := 0.5;
  Vertices.Z := 0.5;
  Vertices.nX := 0.0;
  Vertices.nY := 1.0;
  Vertices.nZ := 0.0;
  Vertices.U := 0.0;
  Vertices.V := 1.0;
  Inc(Vertices);

////////////
  Vertices.X := 0.5;
  Vertices.Y := 0.5;
  Vertices.Z := -0.5;
  Vertices.nX := 1.0;
  Vertices.nY := 0.0;
  Vertices.nZ := 0.0;
  Vertices.U := 1.0;
  Vertices.V := 0.0;
  Inc(Vertices);

  Vertices.X :=  0.5;
  Vertices.Y := -0.5;
  Vertices.Z := -0.5;
  Vertices.nX := 1.0;
  Vertices.nY := 0.0;
  Vertices.nZ := 0.0;
  Vertices.U := 0.0;
  Vertices.V := 0.0;
  Inc(Vertices);

  Vertices.X := 0.5;
  Vertices.Y := 0.5;
  Vertices.Z := 0.5;
  Vertices.nX := 1.0;
  Vertices.nY := 0.0;
  Vertices.nZ := 0.0;
  Vertices.U := 1.0;
  Vertices.V := 1.0;
  Inc(Vertices);

  Vertices.X := 0.5;
  Vertices.Y := 0.5;
  Vertices.Z := 0.5;
  Vertices.nX := 1.0;
  Vertices.nY := 0.0;
  Vertices.nZ := 0.0;
  Vertices.U := 1.0;
  Vertices.V := 1.0;
  Inc(Vertices);

  Vertices.X :=  0.5;
  Vertices.Y := -0.5;
  Vertices.Z := -0.5;
  Vertices.nX := 1.0;
  Vertices.nY := 0.0;
  Vertices.nZ := 0.0;
  Vertices.U := 0.0;
  Vertices.V := 0.0;
  Inc(Vertices);

  Vertices.X :=  0.5;
  Vertices.Y := -0.5;
  Vertices.Z :=  0.5;
  Vertices.nX := 1.0;
  Vertices.nY := 0.0;
  Vertices.nZ := 0.0;
  Vertices.U := 0.0;
  Vertices.V := 1.0;
  Inc(Vertices);

  Vertices.X :=  0.5;
  Vertices.Y := -0.5;
  Vertices.Z := -0.5;
  Vertices.nX := 0.0;
  Vertices.nY := -1.0;
  Vertices.nZ := 0.0;
  Vertices.U := 1.0;
  Vertices.V := 0.0;
  Inc(Vertices);

  Vertices.X := -0.5;
  Vertices.Y := -0.5;
  Vertices.Z := -0.5;
  Vertices.nX := 0.0;
  Vertices.nY := -1.0;
  Vertices.nZ := 0.0;
  Vertices.U := 0.0;
  Vertices.V := 0.0;
  Inc(Vertices);

  Vertices.X :=  0.5;
  Vertices.Y := -0.5;
  Vertices.Z :=  0.5;
  Vertices.nX := 0.0;
  Vertices.nY := -1.0;
  Vertices.nZ := 0.0;
  Vertices.U := 1.0;
  Vertices.V := 1.0;
  Inc(Vertices);

  Vertices.X :=  0.5;
  Vertices.Y := -0.5;
  Vertices.Z :=  0.5;
  Vertices.nX := 0.0;
  Vertices.nY := -1.0;
  Vertices.nZ := 0.0;
  Vertices.U := 1.0;
  Vertices.V := 1.0;
  Inc(Vertices);

  Vertices.X := -0.5;
  Vertices.Y := -0.5;
  Vertices.Z := -0.5;
  Vertices.nX := 0.0;
  Vertices.nY := -1.0;
  Vertices.nZ := 0.0;
  Vertices.U := 0.0;
  Vertices.V := 0.0;
  Inc(Vertices);

  Vertices.X := -0.5;
  Vertices.Y := -0.5;
  Vertices.Z :=  0.5;
  Vertices.nX := 0.0;
  Vertices.nY := -1.0;
  Vertices.nZ := 0.0;
  Vertices.U := 0.0;
  Vertices.V := 1.0;
  Inc(Vertices);

  Vertices.X :=  0.5;
  Vertices.Y := -0.5;
  Vertices.Z :=  0.5;
  Vertices.nX := 0.0;
  Vertices.nY := 0.0;
  Vertices.nZ := 1.0;
  Vertices.U := 0.0;
  Vertices.V := 0.0;
  Inc(Vertices);

  Vertices.X := -0.5;
  Vertices.Y := -0.5;
  Vertices.Z :=  0.5;
  Vertices.nX := 0.0;
  Vertices.nY := 0.0;
  Vertices.nZ := 1.0;
  Vertices.U := 0.0;
  Vertices.V := 1.0;
  Inc(Vertices);

  Vertices.X := 0.5;
  Vertices.Y := 0.5;
  Vertices.Z := 0.5;
  Vertices.nX := 0.0;
  Vertices.nY := 0.0;
  Vertices.nZ := 1.0;
  Vertices.U := 1.0;
  Vertices.V := 0.0;
  Inc(Vertices);

  Vertices.X := 0.5;
  Vertices.Y := 0.5;
  Vertices.Z := 0.5;
  Vertices.nX := 0.0;
  Vertices.nY := 0.0;
  Vertices.nZ := 1.0;
  Vertices.U := 1.0;
  Vertices.V := 0.0;
  Inc(Vertices);

  Vertices.X := -0.5;
  Vertices.Y := -0.5;
  Vertices.Z :=  0.5;
  Vertices.nX := 0.0;
  Vertices.nY := 0.0;
  Vertices.nZ := 1.0;
  Vertices.U := 0.0;
  Vertices.V := 1.0;
  Inc(Vertices);

  Vertices.X := -0.5;
  Vertices.Y :=  0.5;
  Vertices.Z :=  0.5;
  Vertices.nX := 0.0;
  Vertices.nY := 1.0;
  Vertices.nZ := 1.0;
  Vertices.U := 1.0;
  Vertices.V := 1.0;
  Inc(Vertices);

  Vertices.X :=  0.5;
  Vertices.Y :=  0.5;
  Vertices.Z := -0.5;
  Vertices.nX := 0.0;
  Vertices.nY := 0.0;
  Vertices.nZ := -1.0;
  Vertices.U := 0.0;
  Vertices.V := 0.0;
  Inc(Vertices);

  Vertices.X := -0.5;
  Vertices.Y :=  0.5;
  Vertices.Z := -0.5;
  Vertices.nX := 0.0;
  Vertices.nY := 0.0;
  Vertices.nZ := -1.0;
  Vertices.U := 0.0;
  Vertices.V := 1.0;
  Inc(Vertices);

  Vertices.X :=  0.5;
  Vertices.Y := -0.5;
  Vertices.Z := -0.5;
  Vertices.nX := 0.0;
  Vertices.nY := 0.0;
  Vertices.nZ := -1.0;
  Vertices.U := 1.0;
  Vertices.V := 0.0;
  Inc(Vertices);

  Vertices.X :=  0.5;
  Vertices.Y := -0.5;
  Vertices.Z := -0.5;
  Vertices.nX := 0.0;
  Vertices.nY := 0.0;
  Vertices.nZ := -1.0;
  Vertices.U := 1.0;
  Vertices.V := 0.0;
  Inc(Vertices);

  Vertices.X := -0.5;
  Vertices.Y :=  0.5;
  Vertices.Z := -0.5;
  Vertices.nX := 0.0;
  Vertices.nY := 0.0;
  Vertices.nZ := -1.0;
  Vertices.U := 0.0;
  Vertices.V := 1.0;
  Inc(Vertices);

  Vertices.X := -0.5;
  Vertices.Y := -0.5;
  Vertices.Z := -0.5;
  Vertices.nX := 0.0;
  Vertices.nY := 0.0;
  Vertices.nZ := -1.0;
  Vertices.U := 1.0;
  Vertices.V := 1.0;

  Result := FD3DVB.Unlock;
end;

procedure TfrmD3D.DrawScene;
var
  matRotate1 : TD3DMatrix;
  matRotate2 : TD3DMatrix;
begin
  SetRotateZMatrix(matRotate1, -AngleZ);
  SetRotateYMatrix(matRotate2, Angle);

  with FD3DDevice do begin
       SetTransform(D3DTS_WORLD, MatrixMul (matRotate1, matRotate2));

       SetTexture(0, FD3Texture1);
       DrawPrimitive(D3DPT_TRIANGLELIST, 0, 2);

       SetTexture(0, FD3Texture2);
       DrawPrimitive(D3DPT_TRIANGLELIST, 6, 2);

       SetTexture(0, FD3Texture3);
       DrawPrimitive(D3DPT_TRIANGLELIST, 12, 2);

       SetTexture(0, FD3Texture4);
       DrawPrimitive(D3DPT_TRIANGLELIST, 18, 2);

       SetTexture(0, FD3Texture5);
       DrawPrimitive(D3DPT_TRIANGLELIST, 24, 2);

       SetTexture(0, FD3Texture6);
       DrawPrimitive(D3DPT_TRIANGLELIST, 30, 2);

       SetTexture(0, nil);
  end;
end;

function TfrmD3D.InitTexture (var FD3Texture : IDIRECT3DTEXTURE8;
                              const FileName : String) : HRESULT;
var
  hRet : HRESULT;
  d3dlr : TD3DLOCKED_RECT;
  dwDstPitch : DWORD;
  X, Y : DWORD;
  Bmp : tBitmap;
  R, G, B : Byte;
begin
  Bmp := TBitmap.Create;
  Bmp.LoadFromfile (FileName);

  hRet := FD3DDevice.CreateTexture (Bmp.Width, Bmp.Height, 0, 0,
                                    D3DFMT_A8R8G8B8,
                                    D3DPOOL_MANAGED, FD3Texture);
  if FAILED(hRet) then begin
     Result := hRet;
     Exit;
  end;

  hRet := FD3Texture.LockRect(0, d3dlr, nil, 0);
  if FAILED(hRet) then begin
     Result := hRet;
     Exit;
  end;

  dwDstPitch := d3dlr.Pitch;
  for Y := 0 to Bmp.Height - 1 do
      for X := 0 to Bmp.Width - 1 do begin
         R := GetRValue (Bmp.Canvas.Pixels [X, DWORD (Bmp.Height - 1) - Y]);
         G := GetGValue (Bmp.Canvas.Pixels [X, DWORD (Bmp.Height - 1) - Y]);
         B := GetBValue (Bmp.Canvas.Pixels [X, DWORD (Bmp.Height - 1) - Y]);
         PDWORD (DWORD(d3dlr.pBits) + Y * dwDstPitch + X * 4)^ := D3DCOLOR_XRGB(R, G, B);
  end;
  Bmp.Free;

  Result := FD3Texture.UnlockRect(0);
end;

var
  mPos : TPOINT;

procedure TfrmD3D.ReadData;
begin
  GetCursorPos(mPos);

  AngleZ := AngleZ + (320 - mPos.X) / 15;
  Angle := Angle + (240 - mPos.Y) / 15;

  SetCursorPos(320, 240);
end;

end.
